@@ -4,7 +4,7 @@ import re
 from collections import defaultdict
 from time import sleep
 
-from cloudvolume.storage import Storage
+from cloudfiles import CloudFiles
 from cloudvolume import PrecomputedSkeleton
 
 import kimimaro
@@ -27,8 +27,8 @@ class AggregateSkeletonFragmentsOperator(OperatorBase):
             save the merged skeleton file here.
         """
         super().__init__(name=name)
-        self.fragments_storage = Storage(fragments_path)
-        self.output_storage = Storage(output_path)
+        self.fragments_storage = CloudFiles(fragments_path)
+        self.output_storage = CloudFiles(output_path)
    
     def __call__(self, prefix: str):
         logging.info(f'aggregate skeletons with prefix of {prefix}')
@@ -48,7 +48,7 @@ class AggregateSkeletonFragmentsOperator(OperatorBase):
 
         for skl_id, filenames in id2filenames.items():
             logging.info(f'skeleton id: {skl_id}')
-            frags = self.fragments_storage.get_files(filenames)
+            frags = self.fragments_storage.get(filenames)
             frags = [PrecomputedSkeleton.from_precomputed(x['content']) for x in frags]
             skel = PrecomputedSkeleton.simple_merge(frags).consolidate()
             skel = kimimaro.postprocess(
@@ -56,7 +56,7 @@ class AggregateSkeletonFragmentsOperator(OperatorBase):
                 dust_threshold=1000,
                 tick_threshold=3500
             )
-            self.output_storage.put_file(
+            self.output_storage.put(
                 file_path=str(skl_id),
                 content=skel.to_precomputed(),
             )
