@@ -3,17 +3,16 @@ from __future__ import annotations
 import os
 import json
 import time
-from typing import List
 from copy import deepcopy
+from typing import List
 
 import numpy as np
 import h5py
-
 from scipy.spatial import KDTree
 
 import chunkflow
 from chunkflow.chunk import Chunk
-from chunkflow.lib.cartesian_coordinate import BoundingBox, Cartesian
+from chunkflow.lib.cartesian_coordinate import Cartesian, BoundingBox
 
 
 class Synapses():
@@ -301,6 +300,16 @@ class Synapses():
             # transform to C order
             syns.transpose_axis()
         return syns
+    
+    @classmethod
+    def from_file(cls, fname: str, resolution: tuple = None, c_order: bool = True):
+        assert os.path.exists(fname)
+        if fname.endswith('.json'):
+            return cls.from_json(fname, resolution = resolution, c_order=c_order)
+        elif fname.endswith('.h5'):
+            return cls.from_h5(fname, resolution=resolution, c_order=c_order)
+        else:
+            raise ValueError(f'only support JSON and HDF5 file, but got {fname}')
 
     def to_h5(self, fname: str) -> None:
         """save to a HDF5 file
@@ -334,16 +343,74 @@ class Synapses():
             if self.post_users is not None:
                 hf['post_users'] = self.post_users
 
-    @classmethod
-    def from_file(cls, fname: str, resolution: tuple = None, c_order: bool = True):
-        assert os.path.exists(fname)
-        if fname.endswith('.json'):
-            return cls.from_json(fname, resolution = resolution, c_order=c_order)
-        elif fname.endswith('.h5'):
-            return cls.from_h5(fname, resolution=resolution, c_order=c_order)
-        else:
-            raise ValueError(f'only support JSON and HDF5 file, but got {fname}')
+    @property
+    def json_dict(self):
+        """to JSON file
+        """
+        data = {}
+        data['pre'] = self.pre.tolist()
+        if self.post is not None:
+            data['post'] = self.post.tolist()
+        
+        if self.pre_confidence is not None:
+            data['pre_confidence'] = self.pre_confidence.tolist()
+        
+        if self.post_confidence is not None:
+            data['post_confidence'] = self.post_confidence.tolist()
+        
+        return data
 
+    def to_dvid_list_of_dict(self, 
+            user: str = 'chunkflow',
+            comment: str = 'ingested using chunkflow'):
+        """convert to dictionary for bulk ingestion of synapses
+        """
+        data = []
+
+        for post_idx in range(self.post_num):
+            pre_idx = self.post[post_idx, 0]
+            pre_coord = self.pre[pre_idx, :][::-1]
+            pre_coord = [int(x) for x in pre_coord]
+
+            post_coord = self.post_coordinates[post_idx, :][::-1]
+            post_coord = [int(x) for x in post_coord]
+            dic = {
+                'Kind': 'PostSyn',
+                'Pos': post_coord,
+                'Prop': {
+                    'annotation': comment,
+                    'conf': str(self.post_confidence[post_idx]),
+                    'user': user
+                },
+                'Rels': [{'Rel': 'PostSynTo', 'To': pre_coord}],
+                'Tags': []
+            }
+            data.append(dic)
+
+        for pre_idx, post_indices in enumerate(self.pre_index2post_indices):
+            rels = []
+            for post_idx in post_indices:
+                post_coord = self.post_coordinates[post_idx, :][::-1]
+                post_coord = [int(x) for x in post_coord]
+                rels.append({
+                    'Rel': 'PreSynTo', 'To': post_coord
+                })
+
+            pre_coord = self.pre[pre_idx, :][::-1]
+            pre_coord = [int(x) for x in pre_coord]
+            data.append({
+                'Kind': 'PreSyn',
+                'Pos': pre_coord,
+                'Prop': {
+                    'annotation': comment,
+                    'conf': str(self.pre_confidence[pre_idx]),
+                    'user': user
+                },
+                'Rels': rels,
+                'Tags': []
+            })
+        return data
+    
     def to_neutu_task(self, fname: str, 
             software_revision: int=4809,
             description: str = "transformed using chunkflow",
@@ -462,13 +529,13 @@ class Synapses():
     def pre_bounding_box(self) -> BoundingBox:
         bbox = BoundingBox.from_points(self.pre)
         # the end point is exclusive
-        bbox.adjust((0,0,0, 1,1,1))
+        bbox = bbox.adjust((0,0,0, 1,1,1))
         return bbox
 
     def post_bounding_box(self) -> BoundingBox:
         bbox = BoundingBox.from_points(self.post_coordinates)
         # the right direction is exclusive
-        bbox.adjust((0,0,0, 1,1,1))
+        bbox = bbox.adjust((0,0,0, 1,1,1))
         return bbox
     
     @property
