@@ -4,7 +4,10 @@
 import numpy as np
 from chunkflow.chunk.image import Image
 
-def test_image_construction():
+def test_image():
     arr = np.random.randint(256, size=(3,4,5), dtype=np.uint8)
     image = Image(arr, voxel_offset=(-1,-1,-1))
- 
+
+    # test normalization
+    image.normalize_contrast()
+ 
\ No newline at end of file
